@@ -1,4 +1,3 @@
-
 #include <microfeed-common/microfeedconfiguration.h>
 #include <microfeed-common/microfeedmisc.h>
 #include <microfeed-common/microfeedstore.h>
@@ -18,6 +17,8 @@ struct _MicrofeedConfiguration {
 	char* publishers_home_directory;
 	char* subscribers_environment_directory;
 	char* subscribers_home_directory;
+	char* wizards_environment_directory;
+	char* wizards_home_directory;
 	const char** providers;
 	const char** provider_directories;
 	const char** provider_names;
@@ -25,6 +26,10 @@ struct _MicrofeedConfiguration {
 	const char** publishers;
 	const char** publisher_directories;
 	MicrofeedStore* subscriptions;
+	const char** wizards;
+	const char** wizard_directories;
+	const char** wizard_names;
+	const char*** wizard_settings;
 };
 
 typedef struct {
@@ -41,9 +46,12 @@ static void subscription_save(Subscription* subscription);
 static ssize_t read_line(int fd, size_t offset, char* buffer, size_t buffer_size);
 static void get_providers(MicrofeedConfiguration* configuration, const char* directory, unsigned int* length_pointer);
 static void get_publishers(MicrofeedConfiguration* configuration, const char* directory, unsigned int* length_pointer);
+static void get_wizards(MicrofeedConfiguration* configuration, const char* directory, unsigned int* length_pointer);
 static void update_providers(MicrofeedConfiguration* configuration);
 static void update_publishers(MicrofeedConfiguration* configuration);
+static void update_wizards(MicrofeedConfiguration* configuration);
 static int update_and_get_provider_index(MicrofeedConfiguration* configuration, const char* provider_identifier);
+static int update_and_get_wizard_index(MicrofeedConfiguration* configuration, const char* provider_identifier);
 static Subscription* update_and_get_subscription(MicrofeedConfiguration* configuration, const char* application_identifier);
 
 MicrofeedConfiguration* microfeed_configuration_new(void) {
@@ -61,6 +69,9 @@ MicrofeedConfiguration* microfeed_configuration_new(void) {
 	if ((directory = getenv("MICROFEED_SUBSCRIBERS_DIRECTORY"))) {
 		configuration->subscribers_environment_directory = strdup(directory);
 	}
+	if ((directory = getenv("MICROFEED_WIZARDS_DIRECTORY"))) {
+		configuration->wizards_environment_directory = strdup(directory);
+	}
 	if ((directory = getenv("HOME"))) {
 		length = strlen(directory);
 		configuration->providers_home_directory = microfeed_memory_allocate_bytes(length + 22); /* Remember to update. */
@@ -72,6 +83,9 @@ MicrofeedConfiguration* microfeed_configuration_new(void) {
 		configuration->subscribers_home_directory = microfeed_memory_allocate_bytes(length + 24); /* Remember to update. */
 		memcpy(configuration->subscribers_home_directory, directory, length);
 		memcpy(configuration->subscribers_home_directory + length, "/.microfeed/subscribers", 24); /* Remember the NULL byte */
+		configuration->wizards_home_directory = microfeed_memory_allocate_bytes(length + 20); /* Remember to update. */
+		memcpy(configuration->wizards_home_directory, directory, length);
+		memcpy(configuration->wizards_home_directory + length, "/.microfeed/wizards", 20); /* Remember the NULL byte */
 	}
 	configuration->subscriptions = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)subscription_get_application_identifier);
 	
@@ -83,10 +97,12 @@ void microfeed_configuration_free(MicrofeedConfiguration* configuration) {
 	free(configuration->providers_environment_directory);
 	free(configuration->publishers_environment_directory);
 	free(configuration->subscribers_environment_directory);
+	free(configuration->wizards_environment_directory);
 	microfeed_memory_free(configuration->providers_home_directory);
 	microfeed_memory_free(configuration->publishers_home_directory);
 	microfeed_memory_free(configuration->subscribers_home_directory);
 	microfeed_store_free(configuration->subscriptions);
+	microfeed_memory_free(configuration->wizards_home_directory);
 	
 	microfeed_memory_free(configuration);
 }
@@ -127,7 +143,28 @@ void microfeed_configuration_invalidate(MicrofeedConfiguration* configuration) {
 		configuration->publishers = NULL;
 		configuration->publisher_directories = NULL;
 	}
-	microfeed_store_remove_and_free_all(configuration->subscriptions, (MicrofeedStoreFreeDataFunction)subscription_free);	
+	microfeed_store_remove_and_free_all(configuration->subscriptions, (MicrofeedStoreFreeDataFunction)subscription_free);
+	if (configuration->wizards) {
+		for (i = 0; configuration->wizards[i]; i++) {
+			free((void*)configuration->wizards[i]);
+			/* The items in wizard_directories point to strings in configuration (no free). */
+			free((void*)configuration->wizard_names[i]);
+			if (configuration->wizard_settings[i]) {
+				for (k = 0; configuration->wizard_settings[i][k]; k++) {
+					free((void*)configuration->wizard_settings[i][k]);
+				}
+				free(configuration->wizard_settings[i]);
+			}
+		}
+		free(configuration->wizards);
+		free(configuration->wizard_names);
+		free(configuration->wizard_settings);
+		free(configuration->wizard_directories);
+		configuration->wizards = NULL;
+		configuration->wizard_names = NULL;
+		configuration->wizard_settings = NULL;
+		configuration->wizard_directories = NULL;
+	}
 }
 
 const char**  microfeed_configuration_get_providers(MicrofeedConfiguration* configuration) {
@@ -223,20 +260,26 @@ const char** microfeed_configuration_get_subscriptions(MicrofeedConfiguration* c
 
 void microfeed_configuration_add_subscription(MicrofeedConfiguration* configuration, const char* application_identifier, const char* publisher_identifier) {
 	Subscription* subscription;
+	int found;
 	int i;
 	
 	if (!(subscription = microfeed_store_get(configuration->subscriptions, application_identifier, Subscription))) {
 		subscription = update_and_get_subscription(configuration, application_identifier);
 	}
 	if (subscription) {
+		found = 0;
 		for (i = 0; subscription->subscriptions[i]; i++) {
-			/* Nothing */
+			if (!strcmp(subscription->subscriptions[i], publisher_identifier)) {
+				found = 1;
+			}
 		}
-		subscription->subscriptions = (const char**)realloc(subscription->subscriptions, (i + 2) * sizeof(char*));
-		subscription->subscriptions[i] = strdup(publisher_identifier);
-		subscription->subscriptions[i + 1] = NULL;
+		if (!found) {
+			subscription->subscriptions = (const char**)realloc(subscription->subscriptions, (i + 2) * sizeof(char*));
+			subscription->subscriptions[i] = strdup(publisher_identifier);
+			subscription->subscriptions[i + 1] = NULL;
 
-		subscription_save(subscription);
+			subscription_save(subscription);
+		}
 	}
 }
 
@@ -264,6 +307,58 @@ void microfeed_configuration_remove_subscription(MicrofeedConfiguration* configu
 	}
 }
 
+int microfeed_configuration_is_subscribed(MicrofeedConfiguration* configuration, const char* application_identifier, const char* publisher_identifier) {
+	int found = 0;
+	Subscription* subscription;
+	int i;
+	
+	if (!(subscription = microfeed_store_get(configuration->subscriptions, application_identifier, Subscription))) {
+		subscription = update_and_get_subscription(configuration, application_identifier);
+	}
+	if (subscription) {
+		found = 0;
+		for (i = 0; subscription->subscriptions[i]; i++) {
+			if (!strcmp(subscription->subscriptions[i], publisher_identifier)) {
+				found = 1;
+				break;
+			}
+		}
+	}
+
+	return found;
+}
+
+const char** microfeed_configuration_get_wizards(MicrofeedConfiguration* configuration) {
+	if (!configuration->wizards) {
+		update_wizards(configuration);
+	}
+	
+	return configuration->wizards;
+}
+
+const char* microfeed_configuration_get_wizard_name(MicrofeedConfiguration* configuration, const char* publisher_identifier) {
+	const char* name = NULL;
+	int index;
+
+	if ((index = update_and_get_wizard_index(configuration, publisher_identifier)) != -1) {
+		name = configuration->wizard_names[index];
+	}
+
+	return name;
+}
+
+const char** microfeed_configuration_get_wizard_settings(MicrofeedConfiguration* configuration, const char* publisher_identifier) {
+	const char** settings = NULL;
+	int index;
+
+	if ((index = update_and_get_wizard_index(configuration, publisher_identifier)) != -1) {
+		settings = configuration->wizard_settings[index];
+	}
+
+	return settings;
+
+}
+
 static Subscription* subscription_new(const char* application_identifier, const char* filename, const char** subscription_strings) {
 	Subscription* subscription;
 	
@@ -351,8 +446,11 @@ static void get_providers(MicrofeedConfiguration* configuration, const char* dir
 					configuration->provider_directories = (const char**)realloc(configuration->provider_directories, *length_pointer  * sizeof(char*));
 					configuration->provider_names = (const char**)realloc(configuration->provider_names, *length_pointer  * sizeof(char*));
 					configuration->provider_feeds = (const char***)realloc(configuration->provider_feeds, *length_pointer * sizeof(char**));
-					if (i < *length_pointer) {
+					if (i < *length_pointer - 1) {
 						memmove(configuration->providers + i + 1, configuration->providers + i, (*length_pointer - i - 1) * sizeof(char*));
+						memmove(configuration->provider_directories + i + 1, configuration->provider_directories + i, (*length_pointer - i - 1) * sizeof(char*));
+						memmove(configuration->provider_names + i + 1, configuration->provider_names + i, (*length_pointer - i - 1) * sizeof(char*));
+						memmove(configuration->provider_feeds + i + 1, configuration->provider_feeds + i, (*length_pointer - i - 1) * sizeof(char*));
 					}
 					configuration->providers[i] = strdup(dirent->d_name);	
 					configuration->provider_directories[i] = directory;
@@ -384,7 +482,7 @@ static void get_publishers(MicrofeedConfiguration* configuration, const char* di
 					(*length_pointer)++;
 					configuration->publishers = (const char**)realloc(configuration->publishers, (*length_pointer + 1) * sizeof(char*));
 					configuration->publisher_directories = (const char**)realloc(configuration->publisher_directories, *length_pointer * sizeof(char*));
-					if (i < *length_pointer) {
+					if (i < *length_pointer - 1) {
 						memmove(configuration->publishers + i + 1, configuration->publishers + i, (*length_pointer - i - 1) * sizeof(char*));
 						memmove(configuration->publisher_directories + i + 1, configuration->publisher_directories + i, (*length_pointer - i - 1) * sizeof(char*));
 					}
@@ -398,6 +496,44 @@ static void get_publishers(MicrofeedConfiguration* configuration, const char* di
 	}
 }
 
+static void get_wizards(MicrofeedConfiguration* configuration, const char* directory, unsigned int* length_pointer) {
+	DIR* dir;
+	struct dirent* dirent;
+	int i;
+	int order;
+	
+	if ((dir = opendir(directory))) {
+		while ((dirent = readdir(dir))) {
+			if (dirent->d_name[0] != '.') {
+				for (i = 0; i < *length_pointer; i++) {
+					if ((order = strcmp(configuration->wizards[i], dirent->d_name)) >= 0) {
+						break;
+					}
+				}
+				if (i == *length_pointer || order != 0) {
+					(*length_pointer)++;
+					configuration->wizards = (const char**)realloc(configuration->wizards, (*length_pointer + 1) * sizeof(char*));
+					configuration->wizard_directories = (const char**)realloc(configuration->wizard_directories, *length_pointer  * sizeof(char*));
+					configuration->wizard_names = (const char**)realloc(configuration->wizard_names, *length_pointer  * sizeof(char*));
+					configuration->wizard_settings = (const char***)realloc(configuration->wizard_settings, *length_pointer * sizeof(char**));
+					if (i < *length_pointer - 1) {
+						memmove(configuration->wizards + i + 1, configuration->wizards + i, (*length_pointer - i - 1) * sizeof(char*));
+						memmove(configuration->wizard_directories + i + 1, configuration->wizard_directories + i, (*length_pointer - i - 1) * sizeof(char*));
+						memmove(configuration->wizard_names + i + 1, configuration->wizard_names + i, (*length_pointer - i - 1) * sizeof(char*));
+						memmove(configuration->wizard_settings + i + 1, configuration->wizard_settings + i, (*length_pointer - i - 1) * sizeof(char*));
+					}
+					configuration->wizards[i] = strdup(dirent->d_name);	
+					configuration->wizard_directories[i] = directory;
+					configuration->wizard_names[i] = NULL;
+					configuration->wizard_settings[i] = NULL;
+					configuration->wizards[*length_pointer] = NULL;
+				}
+			}
+		}
+		closedir(dir);
+	}
+}
+
 static Subscription* get_subscription(MicrofeedConfiguration* configuration, const char* application_identifier, const char* directory) {
 	Subscription* subscription = NULL;
 	const char** strings = NULL;
@@ -498,6 +634,49 @@ static int update_and_get_provider_index(MicrofeedConfiguration* configuration,
 	return index;
 }
 
+static int update_and_get_wizard_index(MicrofeedConfiguration* configuration, const char* provider_identifier) {
+	int index = -1;
+	char* filename;
+	int difference = -1;
+	int fd;
+	char buffer[1024];
+	size_t offset;
+	ssize_t length;
+	int i;
+
+	if (!configuration->wizards) {
+		update_wizards(configuration);
+	}
+	if (configuration->wizards) {
+		for (index = 0; configuration->wizards[index] && (difference = strcmp(configuration->wizards[index], provider_identifier)) < 0; index++) {
+		}
+		if (!difference) {
+			if (!configuration->wizard_names[index]) {
+				filename = microfeed_util_string_concatenate(configuration->wizard_directories[index], "/", provider_identifier, NULL);
+				if ((fd = open(filename, O_RDONLY)) != -1) {
+					offset = 0;
+					if ((length = read_line(fd, offset, buffer, 1024)) > 0) {
+						configuration->wizard_names[index] = strdup(buffer);
+					}
+					offset += length;
+					for (i = 0; (length = read_line(fd, offset, buffer, 1024)) > 0; i++) {
+						configuration->wizard_settings[index] = (const char**)realloc(configuration->wizard_settings[index], (i + 2) * sizeof(char*));
+						configuration->wizard_settings[index][i] = strdup(buffer);
+						offset += length;
+					}
+					configuration->wizard_settings[index][i] = NULL;
+					close(fd);
+				}
+				free(filename);
+			}
+		} else {
+			index = -1;
+		}
+	}
+
+	return index;
+}
+
 static Subscription* update_and_get_subscription(MicrofeedConfiguration* configuration, const char* application_identifier) {
 	Subscription* subscription = NULL;
 	
@@ -509,3 +688,17 @@ static Subscription* update_and_get_subscription(MicrofeedConfiguration* configu
 		
 	return subscription;
 }
+
+static void update_wizards(MicrofeedConfiguration* configuration) {
+	unsigned int length;
+
+	length = 0;
+	if (configuration->wizards_environment_directory) {
+		get_wizards(configuration, configuration->wizards_environment_directory, &length);
+	}
+	if (configuration->wizards_home_directory) {
+		get_wizards(configuration, configuration->wizards_home_directory, &length);
+	}
+	get_wizards(configuration, "/usr/local/lib/microfeed/wizards", &length);
+	get_wizards(configuration, "/usr/lib/microfeed/wizards", &length);
+}
