@@ -33,7 +33,7 @@ static void no_more_subscribers(MicrofeedPublisher* publisher, void* user_data);
  * @param user_data A pointer to user data that is needed when a new #MicrofeedPublisher is instantiated.
  * @return Instantiated MicrofeedProvider, or NULL if the registration of the bus name failed.
  */ 
-MicrofeedProvider* microfeed_provider_new(const char* bus_name, MicrofeedConfiguration* configuration, DBusConnection* connection, MicrofeedProviderCallbacks* callbacks, void* user_data) {
+MicrofeedProvider* microfeed_provider_new(const char* bus_name, DBusConnection* connection, MicrofeedProviderCallbacks* callbacks, void* user_data) {
 	MicrofeedProvider* provider;
 	DBusError error;
 	
@@ -53,7 +53,7 @@ MicrofeedProvider* microfeed_provider_new(const char* bus_name, MicrofeedConfigu
 		return NULL;
 	} else {
 		provider->bus_name = strdup(bus_name);
-		provider->configuration = configuration;
+		provider->configuration = microfeed_configuration_new();
 		provider->connection = connection;
 		provider->callbacks = *callbacks;
 		provider->user_data = user_data;	
@@ -165,10 +165,16 @@ static DBusHandlerResult handle_message(DBusConnection* connection, DBusMessage*
 			publisher_identifier = microfeed_util_string_concatenate(object_path + strlen(MICROFEED_DBUS_OBJECT_PATH_PREFIX_PUBLISHER), ":", provider->bus_name, NULL);
 			callbacks = provider->callbacks.publisher_callbacks;
 			callbacks.no_more_subscribers = no_more_subscribers;
-			publisher = microfeed_publisher_new(publisher_identifier, publisher_directory, provider->connection, &callbacks, provider->user_data);
-			microfeed_publisher_set_provider(publisher, provider);
-			microfeed_provider_add_publisher(provider, publisher);
-			free(publisher_identifier);
+			if ((publisher = microfeed_publisher_new(publisher_identifier, publisher_directory, provider->connection, &callbacks, provider->user_data))) {
+				microfeed_publisher_set_provider(publisher, provider);
+				microfeed_provider_add_publisher(provider, publisher);
+				free(publisher_identifier);
+			} else {
+				reply = dbus_message_new_error(message, MICROFEED_DBUS_INTERFACE_ERROR "." MICROFEED_ERROR_UNKNOWN, "Instantiation of a publisher failed.");
+				dbus_connection_send(connection, reply, NULL);
+				dbus_message_unref(reply);
+				result = DBUS_HANDLER_RESULT_HANDLED;
+			}
 		} else {
 			reply = dbus_message_new_error(message, MICROFEED_DBUS_INTERFACE_ERROR "." MICROFEED_ERROR_NO_SUCH_PUBLISHER, "Publisher does not exist.");
 			dbus_connection_send(connection, reply, NULL);
