@@ -131,18 +131,18 @@ static DBusObjectPathVTable object_vtable = {
  */
 MicrofeedPublisher* microfeed_publisher_new(const char* publisher_identifier, const char* directory, DBusConnection* connection, MicrofeedPublisherCallbacks* callbacks, void* user_data) {
 	MicrofeedPublisher* publisher;
-	char* colon;
-	
+	char* separator;
+printf("%s\n", publisher_identifier);	
 	publisher = microfeed_memory_allocate(MicrofeedPublisher);
 	publisher->identifier = strdup(publisher_identifier);
-	if (!(colon = strchr(publisher->identifier, ':')) || colon[1] == 0) {
+	if (!(separator = strchr(publisher->identifier, MICROFEED_PUBLISHER_IDENTIFIER_SEPARATOR_CHAR)) || separator[1] == 0) {
 		free(publisher->identifier);
 		microfeed_memory_free(publisher);
 		publisher = NULL;
 	} else {
-		*colon = 0;
+		*separator = 0;
 		publisher->object_path = microfeed_util_string_concatenate(MICROFEED_DBUS_OBJECT_PATH_PREFIX_PUBLISHER, publisher->identifier, NULL);
-		*colon = ':';
+		*separator = MICROFEED_PUBLISHER_IDENTIFIER_SEPARATOR_CHAR;
 		publisher->connection = connection;
 		publisher->callbacks = *callbacks;
 		publisher->user_data = user_data;
@@ -152,11 +152,13 @@ MicrofeedPublisher* microfeed_publisher_new(const char* publisher_identifier, co
 		publisher->image_properties[2] = 0;
 
 		if (!dbus_connection_register_object_path(connection, publisher->object_path, &object_vtable, publisher)) {
+printf("dbus\n", publisher_identifier);	
 		
 			return NULL;
 		}
 		
 		if (!(publisher->database_environment = microfeed_database_environment_new(publisher_identifier, directory))) {
+printf("database\n", publisher_identifier);	
 			
 			return NULL;
 		}
@@ -183,12 +185,26 @@ MicrofeedPublisher* microfeed_publisher_new(const char* publisher_identifier, co
 /**
  * Frees the resources allocated for the publisher.
  * 
- * TODO: This function is not yet implemented.
- *
  * @param publisher Instantiated MicrofeedPublisher.
  */
 void microfeed_publisher_free(MicrofeedPublisher* publisher) {
-	
+	dbus_connection_unregister_object_path(publisher->connection, publisher->object_path);
+	microfeed_database_environment_free(publisher->database_environment);
+	microfeed_feed_remove_subscriber(publisher->settings_feed, NULL);
+	microfeed_feed_remove_subscriber(publisher->images_feed, NULL);
+
+	free(publisher->identifier);
+	free(publisher->object_path);
+	free(publisher->image_properties);
+	microfeed_store_free(publisher->subscribers);
+	microfeed_store_free(publisher->feeds);
+/* TODO
+	microfeed_feed_unref(publisher->settings_feed);
+	microfeed_feed_unref(publisher->images_feed);
+*/
+	microfeed_thread_free(publisher->singleton_update_thread);
+/* TODO	microfeed_thread_pool_free(publisher->thread_pool); */
+	microfeed_store_free(publisher->timeouts);
 }
 
 /**
