@@ -16,6 +16,7 @@ struct _MicrofeedSubscriber {
 	DBusConnection* connection;
 	char* object_path;
 	MicrofeedConfiguration* configuration;
+	unsigned int max_retries;
 	
 	MicrofeedStore* publishers_by_identifier;
 	MicrofeedStore* providers_by_bus_name;
@@ -56,6 +57,8 @@ typedef struct {
 	Publisher* publisher;
 	char* uri;
 	char* uid;
+	DBusMessage* message;
+	unsigned int retry_counter;
 	MicrofeedSubscriberErrorCallback callback;
 	void* user_data;
 } MethodReturnData;
@@ -116,14 +119,15 @@ static DBusObjectPathVTable object_vtable = {
 	object_message
 };	
 
-MicrofeedSubscriber* microfeed_subscriber_new(const char* identifier, const char* object_path, MicrofeedConfiguration* configuration, DBusConnection* connection) {
+MicrofeedSubscriber* microfeed_subscriber_new(const char* identifier, const char* object_path, DBusConnection* connection) {
 	MicrofeedSubscriber* subscriber;
 	
 	subscriber = microfeed_memory_allocate(MicrofeedSubscriber);
 	subscriber->identifier = strdup(identifier);
 	subscriber->connection = connection;
 	subscriber->object_path = strdup(object_path);
-	subscriber->configuration = configuration;
+	subscriber->configuration = microfeed_configuration_new();
+	subscriber->max_retries = 3;
 	
 	subscriber->publishers_by_identifier = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp,
 	                                                                  (MicrofeedStoreGetKeyFunction)publisher_get_identifier);
@@ -484,23 +488,8 @@ static Provider* provider_new(MicrofeedSubscriber* subscriber, const char* bus_n
 }
 
 static void provider_free(Provider* provider) {
-	char buffer[1024];
-
 	microfeed_store_remove(provider->subscriber->providers_by_bus_name, provider);
-	if (provider->unique_connection_name) {
-		microfeed_store_remove(provider->subscriber->providers_by_unique_connection_name, provider);
-	
-		snprintf(buffer, 1024, "type='signal',sender='org.freedesktop.DBus',interface='org.freedesktop.DBus',member='NameOwnerChanged',arg0='%s'", provider->unique_connection_name);
-		dbus_bus_remove_match(provider->subscriber->connection, buffer, NULL);
-		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_PUBLISHER);
-		dbus_bus_remove_match(provider->subscriber->connection, buffer, NULL);
-		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s',destination='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION, dbus_bus_get_unique_name(provider->subscriber->connection));
-		dbus_bus_remove_match(provider->subscriber->connection, buffer, NULL);
-		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_ERROR);
-		dbus_bus_remove_match(provider->subscriber->connection, buffer, NULL);
-		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s',destination='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_ERROR_TO_DESTINATION, dbus_bus_get_unique_name(provider->subscriber->connection));
-		dbus_bus_remove_match(provider->subscriber->connection, buffer, NULL);
-	}
+	provider_set_unique_connection_name(provider, NULL);
 
 	free(provider->bus_name);
 	free(provider->unique_connection_name);
@@ -524,40 +513,59 @@ static const char* provider_get_unique_connection_name(Provider* provider) {
 
 static void provider_set_unique_connection_name(Provider* provider, const char* unique_connection_name) {
 	char buffer[1024];
+printf("sucn: %s\n", unique_connection_name);
+	if (unique_connection_name) {
+		if (provider->unique_connection_name) {
+			provider_set_unique_connection_name(provider, NULL);
+		}
+		provider->unique_connection_name = strdup(unique_connection_name);
+		microfeed_store_insert(provider->subscriber->providers_by_unique_connection_name, provider);
 
-	provider->unique_connection_name = strdup(unique_connection_name);
-	microfeed_store_insert(provider->subscriber->providers_by_unique_connection_name, provider);
+		snprintf(buffer, 1024, "type='signal',sender='org.freedesktop.DBus',interface='org.freedesktop.DBus',member='NameOwnerChanged',arg0='%s'", provider->unique_connection_name);
+		dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
+		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_PUBLISHER);
+		dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
+		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s',destination='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION, dbus_bus_get_unique_name(provider->subscriber->connection));
+		dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
+		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_ERROR);
+		dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
+		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s',destination ='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_ERROR_TO_DESTINATION, dbus_bus_get_unique_name(provider->subscriber->connection));
+		dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
+	} else if (provider->unique_connection_name) {
+		snprintf(buffer, 1024, "type='signal',sender='org.freedesktop.DBus',interface='org.freedesktop.DBus',member='NameOwnerChanged',arg0='%s'", provider->unique_connection_name);
+		dbus_bus_remove_match(provider->subscriber->connection, buffer, NULL);
+		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_PUBLISHER);
+		dbus_bus_remove_match(provider->subscriber->connection, buffer, NULL);
+		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s',destination='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION, dbus_bus_get_unique_name(provider->subscriber->connection));
+		dbus_bus_remove_match(provider->subscriber->connection, buffer, NULL);
+		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_ERROR);
+		dbus_bus_remove_match(provider->subscriber->connection, buffer, NULL);
+		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s',destination='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_ERROR_TO_DESTINATION, dbus_bus_get_unique_name(provider->subscriber->connection));
 
-	snprintf(buffer, 1024, "type='signal',sender='org.freedesktop.DBus',interface='org.freedesktop.DBus',member='NameOwnerChanged',arg0='%s'", provider->unique_connection_name);
-	dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
-	snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_PUBLISHER);
-	dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
-	snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s',destination='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION, dbus_bus_get_unique_name(provider->subscriber->connection));
-	dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
-	snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_ERROR);
-	dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
-	snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s',destination ='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_ERROR_TO_DESTINATION, dbus_bus_get_unique_name(provider->subscriber->connection));
-	dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
+		microfeed_store_remove(provider->subscriber->providers_by_unique_connection_name, provider);
+		free(provider->unique_connection_name);
+		provider->unique_connection_name = NULL;	
+	}
 }
 
 static Publisher* publisher_new(Provider* provider, const char* identifier) {
 	Publisher* publisher = NULL;
-	char* colon;
+	char* separator;
 	
 	publisher = microfeed_memory_allocate(Publisher);
 	publisher->reference_count = 1;
 	publisher->identifier = strdup(identifier);
-	if ((colon = strchr(publisher->identifier, ':'))) {
-		*colon = 0;
+	if ((separator = strchr(publisher->identifier, MICROFEED_PUBLISHER_IDENTIFIER_SEPARATOR_CHAR))) {
+		*separator = 0;
 		publisher->object_path = microfeed_util_string_concatenate(MICROFEED_DBUS_OBJECT_PATH_PREFIX_PUBLISHER, publisher->identifier, NULL);
-		*colon = ':';
+		*separator = MICROFEED_PUBLISHER_IDENTIFIER_SEPARATOR_CHAR;
 	} else {
 		publisher->object_path = strdup(identifier);
 	}
 	publisher->feeds = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp,
 		                                      (MicrofeedStoreGetKeyFunction)feed_get_uri);
 	publisher->provider = provider;
-		
+
 	microfeed_store_insert(provider->publishers_by_object_path, publisher);
 	microfeed_store_insert(provider->subscriber->publishers_by_identifier, publisher);
 	
@@ -768,7 +776,7 @@ static Publisher* get_publisher(MicrofeedSubscriber* subscriber, const char* pub
 	Provider* provider;
 	
 	if (!(publisher = microfeed_store_get(subscriber->publishers_by_identifier, publisher_identifier, Publisher))) {
-		if ((bus_name = strchr(publisher_identifier, ':')) && bus_name[1] != 0) {
+		if ((bus_name = strchr(publisher_identifier, MICROFEED_PUBLISHER_IDENTIFIER_SEPARATOR_CHAR)) && bus_name[1] != 0) {
 			bus_name++;
 			if (!(provider = microfeed_store_get(subscriber->providers_by_bus_name, bus_name, Provider))) {
 				microfeed_configuration_invalidate(subscriber->configuration);
@@ -882,6 +890,8 @@ static void handle_publisher_method_return(DBusPendingCall* pending, void* user_
 	DBusError error;
 	const char* error_name;
 	const char* error_message;
+	int retrying = 0;
+	DBusPendingCall* pending_call;
 	
 	data = (MethodReturnData*)user_data;
 	reply = dbus_pending_call_steal_reply(pending);
@@ -890,7 +900,18 @@ static void handle_publisher_method_return(DBusPendingCall* pending, void* user_
 			data->callback(data->publisher->provider->subscriber, data->publisher->identifier, data->uri, data->uid, MICROFEED_ERROR_DBUS_MESSAGE_FAILED, "No reply from the publisher.", data->user_data);
 		}
 	} else if (dbus_message_get_type(reply) == DBUS_MESSAGE_TYPE_ERROR) {
-		if (data->callback) {
+		if (data->retry_counter < data->publisher->provider->subscriber->max_retries) {
+			retrying = 1;
+			if (data->retry_counter) {
+				/* TODO: Some pause would be nice, but sleep(data->retry_counter); is not possible. */
+			}
+			data->retry_counter++;
+			
+			dbus_connection_send_with_reply(data->publisher->provider->subscriber->connection, data->message, &pending_call, -1);
+			if (pending_call) {
+				dbus_pending_call_set_notify(pending_call, handle_publisher_method_return, data, NULL);
+			}
+		} else if (data->callback) {
 			if (!(error_name = dbus_message_get_error_name(reply))) {
 				error_name = MICROFEED_ERROR_UNKNOWN;
 			}
@@ -912,10 +933,13 @@ static void handle_publisher_method_return(DBusPendingCall* pending, void* user_
 		dbus_message_unref(reply);
 	}
 		
-	publisher_unref(data->publisher);	
-	free(data->uri);
-	free(data->uid);
-	microfeed_memory_free(data);		
+	if (!retrying) {
+		dbus_message_unref(data->message);
+		publisher_unref(data->publisher);	
+		free(data->uri);
+		free(data->uid);
+		microfeed_memory_free(data);		
+	}
 }
 
 static void call_publisher_method(Publisher* publisher, const char* uri, const char* uid, DBusMessage* message, MicrofeedSubscriberErrorCallback callback, void* user_data) {
@@ -925,6 +949,7 @@ static void call_publisher_method(Publisher* publisher, const char* uri, const c
 	dbus_connection_send_with_reply(publisher->provider->subscriber->connection, message, &pending_call, -1);
 	if (pending_call) {
 		data = microfeed_memory_allocate(MethodReturnData);
+		data->message = dbus_message_copy(message);
 		data->publisher = publisher_ref(publisher);
 		if (uri) {
 			data->uri = strdup(uri);
@@ -974,12 +999,15 @@ static DBusHandlerResult object_message(DBusConnection* connection, DBusMessage*
 			for (feed_iterator = microfeed_store_iterate(publisher->feeds, NULL);
 			     (feed = microfeed_store_iterator_get(feed_iterator, Feed));
 			     microfeed_store_iterator_next(feed_iterator)) {
-				feed->callbacks.error_occured(subscriber, publisher->identifier, feed->uri, NULL, MICROFEED_ERROR_PROVIDER_CLOSED_CONNECTION, "Provider closed connection", feed->user_data);	
-				feed_free(feed);
+				if (feed->callbacks.error_occured) {
+					feed->callbacks.error_occured(subscriber, publisher->identifier, feed->uri, NULL, MICROFEED_ERROR_PROVIDER_CLOSED_CONNECTION, "Provider closed connection", feed->user_data);	
+				}
+				/* TODO: Think! Think! Think! feed_free(feed); */
 			}
 			microfeed_store_iterator_free(feed_iterator);
-			publisher_unref(publisher);
+			/* TODO: I am quite sure that this should not be here: publisher_unref(publisher); */
 		}
+		provider_set_unique_connection_name(provider, NULL);
 		microfeed_store_iterator_free(publisher_iterator);
 	}
 
