@@ -56,7 +56,7 @@ typedef struct {
 	MicrofeedSubscriberItemDataCallback item_data_received;
 } MicrofeedSubscriberCallbacks;
 
-MicrofeedSubscriber* microfeed_subscriber_new(const char* identifier, const char* object_path, MicrofeedConfiguration* configuration, DBusConnection* connection);
+MicrofeedSubscriber* microfeed_subscriber_new(const char* identifier, const char* object_path, DBusConnection* connection);
 void microfeed_subscriber_free(MicrofeedSubscriber* subscriber);
 
 const char* microfeed_subscriber_get_identifier(MicrofeedSubscriber* subscriber);
@@ -73,7 +73,6 @@ int microfeed_subscriber_send_item_data(MicrofeedSubscriber* subscriber, const c
 int microfeed_subscriber_mark_item(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
 int microfeed_subscriber_unmark_item(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
 int microfeed_subscriber_read_items(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, const char* start_uid, const char* end_uid, MicrofeedSubscriberReplyCallback callback, void* user_data);
-MicrofeedConfiguration* microfeed_subscriber_get_configuration(MicrofeedSubscriber* subscriber);
 int microfeed_subscriber_handle_configured_subscriptions(MicrofeedSubscriber* subscriber, MicrofeedSubscriberConfiguredSubscribeCallback configured_subscribe, MicrofeedSubscriberConfiguredUnsubscribeCallback configured_unsubscribe, void* user_data);
 
 /** @} */ 
